'***********************************************************************
'*  Many thanks to Timothy Box for creating the original INT_CTRL.PBP  *
'***********************************************************************
'*  Name    : INT_CTRL-18F.inc        Version 2.0A                     *
'*  Author  : Darrel Taylor & John Barrat                              *
'*  Target  : PIC18Fxxx                                                *
'*  Date    : 20/11/04                                                 *
'*  Notes: This interface will will set up the vectors to interrupt    *
'*         subroutines.  After saving the context registers it will    *
'*         then pass control to your P+ interrupt routine.             *
'*         After your routine is finished, it will                     *
'*         restore P+ system vars and return from the interrupt.       *
'*                                                                     *
'*  Defines:                                                           *
'*     TMR1_HANDLER         Assigns Tmr1 Interrupt subroutine          *
'*        TMR1_FREQ         defaults to 100 if not defined             *
'*     RX_HANDLER           USART Receive Interrupt                    *
'*     TX_HANDLER           USART Transmit Interrupt                   *
'*     SSP_HANDLER          Synchronous Serial Port Interrupt          *
'*     AD_HANDLER           A/D Converter Interrupt                    *
'*	   LVD_HANDLER			Low voltage Detect Interrupt			   *
'*     IOC_HANDLER          Interrupt on Change (PortB 4-7)            *
'*     INT0_HANDLER         Interrupt on PortB 0                       *
'*     ELSE_INT_HANDLER     All other interrupts                       *
'*                                                                     *
'***********************************************************************
'Device 18F252
Symbol IPEN  = RCON.7    	 	 ' Interrupt Priority Enable bit
Symbol GIE   = INTCON.7   	     ' Global Interrupt Enable   (If IPEN = 0)

GoTo OVER_INT_CODE

Interrupt_Init:
	IPEN = 0                     ' Set Interrupt priority to Off
	While GIE = 1                ' Make sure they are off 
	GIE = 0                      ' Continue to clear GIE 
	Wend                         ' Exit when GIE is clear
	
	ASM
	; These are local Vars used 
	EnableGlobal = 0
	EnablePeripheral = 0 
	
	#IFDEF TMR1_HANDLER
		#IFNDEF TMR1_FREQ
 		TMR1_FREQ = 100
		#ENDIF
		EnableGlobal = 1
		EnablePeripheral = 1
		clrf  T1CON 
		bcf	  PIR1,	  0           ; Clear Interrupt Flag
	    bsf   T1CON,  0			  ; Turn on Timer1, prescaler = 1
	    bsf   PIE1,   0    		  ; Enable TMR1 overflow interrupt
	#ENDIF
	    
	#IFDEF RX_HANDLER
 	   EnableGlobal = 1
	   EnablePeripheral = 1 
	   bcf    PIE1,   5      	  ; Clear RX Interrupt (During Initialisation)
	#ENDIF
	
	#IFDEF TX_HANDLER
	   EnableGlobal = 1
	   EnablePeripheral = 1
	   bcf    PIE1,   4	   		  ; Clear TX Interupt flag (During Initialisation)
	#ENDIF
	
	#IFDEF MSSP_HANDLER           ; MSSP Port
	   EnableGlobal = 1
	   EnablePeripheral = 1 
	#ENDIF
	
	#IFDEF AD_HANDLER             ; A/D Converter
	   EnableGlobal = 1
	   EnablePeripheral = 1 
	#ENDIF
	
	#IFDEF LVD_HANDLER            ; Low Voltage Detection
	   ;Set up the Low Voltage Detection 
	   movlw  0x1B 				  ; Enable LVD, Trip point = 4.2V
	   movwf  LVDCON ;
	LVD_STABLE
	   btfss  LVDCON, IRVST 	  ; Has LVD circuitry stabilized?
	   GoTo   LVD_STABLE 		  ; NO, Wait longer
	   bcf    PIR2,   LVDIF 	  ; YES, clear LVD interrupt flag
	   bsf    PIE2,   LVDIE 	  ; Enable LVD interrupt
	   EnableGlobal = 1
	   EnablePeripheral = 1
	#ENDIF
	
	#IFDEF IOC_HANDLER            ; Interrupt on Change
	   EnableGlobal = 1
	   bsf    INTCON, 3           ; Enable the Interrupt on change
	#ENDIF
	
	#IFDEF INT0_HANDLER
	   EnableGlobal = 1
	   bsf    INTCON, 4           ; Enable INT0
	   bsf 	  INTCON2,6           ; Set Edge trigger to Rising
	#ENDIF
	
	#IF EnableGlobal == 1
	   bsf    INTCON, 7           ; Enable global interrupts (7)
	#ENDIF
	
	#IF EnablePeripheral == 1
	   bsf    INTCON, 6           ; Enable peripheral interrupts (6)
	#ENDIF
	ENDASM
	On_Interrupt GoTo INT_CODE
Return

' End of Initialisation ------------------------------------------------------------------------

' Interrupt Routines
' These routines simply detect the type of interrupt received and redirect the program to the
' relevant Interupt service routine.  All routines should return to INT_Code_Cont to allow the
' code to establish that no further interrupts have arrived and to restore the previous context.
' It is the responsibility of the interrupt handlers to clear any hardware flags associated with
' the interrupt.

' IMPORTANT - ALL INTERUPT HANDLERS SHOULD BE KEPT SHORT AND QUICK

INT_CODE:
ASM
; Power Down Interrupt - no need to save registers
#IFDEF LVD_HANDLER          	    ; Low Voltage Detection Interrupt being used
   btfsc  PIR2, LVDIF,0             ; Is LVD Int Set? 
   GoTo   LVD_HANDLER               ; Yes Goto Service Routine - will not be returning!!!
#ENDIF
ENDASM

Context SAVE                        ; Save Registers:

INT_Code_Cont:                      ; Return here from ISRs to catch other interrupts
ASM

; -- Interrupt on Change Interrupt -0-----------  (must be cleared in software)
#IFDEF IOC_HANDLER          		; If Interrupt on Change being used
    btfsc   INTCON, 0               ; Is IOC Interrupt flag set
    GoTo 	IOC_HANDLER             ; Yes - Goto the Interrupt on Change Service Routine
#ENDIF

; ----------  Timer 1 -------------
#IFDEF TMR1_HANDLER                 ; If Timer1 function being used
Ticks = (XTAL*1000000/4)/TMR1_FREQ   ; These are adapted from Tim's "Timer calc.xls"
OSCcalc = 1000000/(XTAL*1000000/Ticks); they are only valid between 16hz and 10,000hz
TMR1_Val = ((65536)-(XTAL*OSCcalc)+7)
    btfss   PIR1, 0                 ; is TMR1IF set?   Timer1 Interrupt Flag
    GoTo    NoTimerInt              ; No.  Bypass timer load
                                    ; Yes, then reload Timer1
    bcf     T1CON,0            		; Turn off timer
    movlw   Low(TMR1_Val)           ;  
    addwf   TMR1L,F                 ; reload timer with correct value
    btfsc   STATUS,C                ; 
    incf    TMR1H,F                 ; 
    movlw   High(TMR1_Val)          ;
    addwf   TMR1H,F                 ; 
    bsf     T1CON,0           		; Turn it back on
    GoTo    TMR1_HANDLER            ; GoTo Service Routine
NoTimerInt    
#ENDIF 
   
; ----------  A/D Convertor   -6---------  
#IFDEF AD_HANDLER            	    ; If AD Interrupt being used
    btfsc   PIR1,6               	; is ADIF set?  A/D Converter Interrupt Flag bit
    GoTo    AD_HANDLER              ; Yes.  Goto Service Routine
#ENDIF
    
; ----------  USART Receive   -5---------  
#IFDEF RX_HANDLER                   ; If USART RX Interrupt being used
    btfsc   PIR1,5               	; is RCIF set?  USART Receive Interrupt Flag
    GoTo    RX_HANDLER              ; Yes GoTo RX Service Routine
#ENDIF
    
; ----------  USART Transmit  -4---------  
#IFDEF TX_HANDLER                   ; If USART TX Interrupt being used
    btfsc 	PIE1, 4					; Interrupt has to be enabled
    btfss 	PIR1, 4                 ; and RX Int flag set to be valid RX Interrupt
    GoTo 	NoTXInt                 ; Its not so on to next interrupt
    GoTo 	TX_Handler              ; Both set so Goto TX Service Routine
NoTXInt
#ENDIF
	  
; -- Master Synchronous Serial Port -3---------  (must be cleared in software)
#IFDEF MSSP_HANDLER             	; If SSP Port interrupt being used
    btfsc   PIR1, 3              	; is SSPIF set?  MSSP Interrupt Flag bit
    GoTo    SSP_HANDLER             ; Yes.  Goto the MSSP Service Routine
#ENDIF

; -- INT0 Hardware Interrupt -1-----------------  (must be cleared in software)
#IFDEF INT0_HANDLER                 ; If INT0 interrupt being used
    btfsc   INTCON, 1               ; Is INT0 flag set?
    GoTo    INT0_HANDLER            ; Yes - Goto the INT0 Service Routine
#ENDIF
; ---------------------------------------------------------------------------------------------
ENDASM

INT_RETURN:
Context Restore                     ; Restore Registers back state prior to interrrupt

OVER_INT_CODE:
